@@ -4,8 +4,9 @@ import numpy as np
 import cv2
 import tifffile
 import logging, pathlib, sys
+from pathlib import Path
 
-from . import utils, plot, transforms
+from . import utils, plot, transforms, omnipose
 
 try:
     from PyQt5 import QtGui, QtCore, Qt, QtWidgets
@@ -50,6 +51,11 @@ def logger_setup():
 
     return logger, log_file
 
+# helper function to check for a path; if it doesn't exist, make it 
+def check_dir(path):
+    if not os.path.isdir(path):
+        os.mkdir(path)
+
 def outlines_to_text(base, outlines):
     with open(base + '_cp_outlines.txt', 'w') as f:
         for o in outlines:
@@ -78,7 +84,10 @@ def imsave(filename, arr):
     if ext== '.tif' or ext=='tiff':
         tifffile.imsave(filename, arr)
     else:
+        if len(arr.shape)>2:
+            arr = cv2.cvtColor(arr, cv2.COLOR_BGR2RGB)
         cv2.imwrite(filename, arr)
+#         skimage.io.imsave(filename, arr.astype()) #cv2 doesn't handle transparency
 
 def get_image_files(folder, mask_filter, imf=None, look_one_level_down=False):
     """ find all images in a folder and if look_one_level_down all subfolders """
@@ -130,6 +139,7 @@ def get_label_files(image_names, mask_filter, imf=None):
     else:
         flow_names = [label_names[n] + '_flows.tif' for n in range(nimg)]
     if not all([os.path.exists(flow) for flow in flow_names]):
+        print('Not all flows are present. Run flow generation again.')
         flow_names = None
     
     # check for masks
@@ -145,8 +155,8 @@ def get_label_files(image_names, mask_filter, imf=None):
     return label_names, flow_names
 
 
-def load_train_test_data(train_dir, test_dir=None, image_filter=None, mask_filter='_masks', unet=False):
-    image_names = get_image_files(train_dir, mask_filter, imf=image_filter)
+def load_train_test_data(train_dir, test_dir=None, image_filter=None, mask_filter='_masks', unet=False, look_one_level_down=True):
+    image_names = get_image_files(train_dir, mask_filter, image_filter, look_one_level_down)
     nimg = len(image_names)
     images = [imread(image_names[n]) for n in range(nimg)]
 
@@ -165,7 +175,7 @@ def load_train_test_data(train_dir, test_dir=None, image_filter=None, mask_filte
     # testing data
     test_images, test_labels, image_names_test = None, None, None
     if test_dir is not None:
-        image_names_test = get_image_files(test_dir, mask_filter, imf=image_filter)
+        image_names_test = get_image_files(test_dir, mask_filter, image_filter, look_one_level_down)
         label_names_test, flow_names_test = get_label_files(image_names_test, mask_filter, imf=image_filter)
         nimg = len(image_names_test)
         test_images = [imread(image_names_test[n]) for n in range(nimg)]
@@ -215,6 +225,8 @@ def masks_flows_to_seg(images, masks, flows, diams, file_names, channels=None):
         channels = [0,0]
     
     if isinstance(masks, list):
+        if not isinstance(diams, (list, np.ndarray)):
+            diams = diams * np.ones(len(masks), np.float32)
         for k, [image, mask, flow, diam, file_name] in enumerate(zip(images, masks, flows, diams, file_names)):
             channels_img = channels
             if channels_img is not None and len(channels) > 2:
@@ -251,7 +263,7 @@ def masks_flows_to_seg(images, masks, flows, diams, file_names, channels=None):
                         'masks': masks.astype(np.uint16) if outlines.max()<2**16-1 else masks.astype(np.uint32),
                         'chan_choose': channels,
                         'img': images,
-                        'ismanual': np.zeros(masks.max(), np.bool),
+                        'ismanual': np.zeros(masks.max(), bool),
                         'filename': file_names,
                         'flows': flowi,
                         'est_diam': diams})
@@ -263,7 +275,7 @@ def masks_flows_to_seg(images, masks, flows, diams, file_names, channels=None):
                         'outlines': outlines.astype(np.uint16) if outlines.max()<2**16-1 else outlines.astype(np.uint32),
                      'masks': masks.astype(np.uint16) if masks.max()<2**16-1 else masks.astype(np.uint32),
                      'chan_choose': channels,
-                     'ismanual': np.zeros(masks.max(), np.bool),
+                     'ismanual': np.zeros(masks.max(), bool),
                      'filename': file_names,
                      'flows': flowi,
                      'est_diam': diams})    
@@ -276,8 +288,10 @@ def save_to_png(images, masks, flows, file_names):
     """
     save_masks(images, masks, flows, file_names, png=True)
 
-def save_masks(images, masks, flows, file_names, 
-               png=True, tif=False, outlines=False):
+# Now saves flows, masks, etc. to separate folders.
+def save_masks(images, masks, flows, file_names, png=True, tif=False, channels=[0,0],
+               suffix='',save_flows=False, save_outlines=False, save_ncolor=False, 
+               dir_above=False, in_folders=False, savedir=None, save_txt=True, omni=True):
     """ save masks + nicely plotted segmentation image to png and/or tiff
 
     if png, masks[k] for images[k] are saved to file_names[k]+'_cp_masks.png'
@@ -302,17 +316,53 @@ def save_masks(images, masks, flows, file_names,
 
     file_names: (list of) str
         names of files of images
+        
+    savedir: str
+        absolute path where images will be saved. Default is none (saves to image directory)
+    
+    save_flows, save_outlines, save_ncolor, save_txt: bool
+        Can choose which outputs/views to save.
+        ncolor is a 4 (or 5, if 4 takes too long) index version of the labels that
+        is way easier to visualize than having hundreds of unique colors that may
+        be similar and touch. Any color map can be applied to it (0,1,2,3,4,...).
     
     """
-    
+
     if isinstance(masks, list):
         for image, mask, flow, file_name in zip(images, masks, flows, file_names):
-            save_masks(image, mask, flow, file_name, png=png, tif=tif, outlines=outlines)
+            save_masks(image, mask, flow, file_name, png=png, tif=tif, suffix=suffix,dir_above=dir_above,
+                       save_flows=save_flows,save_outlines=save_outlines,save_ncolor=save_ncolor,
+                       savedir=savedir,save_txt=save_txt,in_folders=in_folders, omni=omni)
         return
     
     if masks.ndim > 2 and not tif:
         raise ValueError('cannot save 3D outputs as PNG, use tif option instead')
-    base = os.path.splitext(file_names)[0]
+#     base = os.path.splitext(file_names)[0]
+    
+    if savedir is None: 
+        if dir_above:
+            savedir = Path(file_names).parent.parent.absolute() #go up a level to save in its own folder
+        else:
+            savedir = Path(file_names).parent.absolute()
+    
+    check_dir(savedir) 
+            
+    basename = os.path.splitext(os.path.basename(file_names))[0]
+    if in_folders:
+        maskdir = os.path.join(savedir,'masks')
+        outlinedir = os.path.join(savedir,'outlines')
+        txtdir = os.path.join(savedir,'txt_outlines')
+        ncolordir = os.path.join(savedir,'ncolor_masks')
+        flowdir = os.path.join(savedir,'flows')
+    else:
+        maskdir = savedir
+        outlinedir = savedir
+        txtdir = savedir
+        ncolordir = savedir
+        flowdir = savedir
+        
+    check_dir(maskdir) 
+
     exts = []
     if masks.ndim > 2 or masks.max()>2**16-1:
         png = False
@@ -322,15 +372,15 @@ def save_masks(images, masks, flows, file_names,
     if tif:
         exts.append('.tif')
 
-    # convert to uint16 if possible so can save as PNG if needed
-    masks = masks.astype(np.uint16) if masks.max()<2**16-1 else masks.astype(np.uint32)
-    
+    # format_labels will also automatically use lowest bit depth possible
+    masks = omnipose.utils.format_labels(masks) 
+
     # save masks
     with warnings.catch_warnings():
         warnings.simplefilter("ignore")
         for ext in exts:
-            imsave(base + '_cp_masks' + ext, masks)
-
+            imsave(os.path.join(maskdir,basename + '_cp_masks' + suffix + ext), masks)
+            
     if png and MATPLOTLIB and not min(images.shape) > 3:
         img = images.copy()
         if img.ndim<3:
@@ -339,15 +389,47 @@ def save_masks(images, masks, flows, file_names,
             np.transpose(img, (1,2,0))
         
         fig = plt.figure(figsize=(12,3))
-        # can save images (set save_dir=None if not)
-        plot.show_segmentation(fig, img, masks, flows[0])
-        fig.savefig(base + '_cp_output.png', dpi=300)
+        plot.show_segmentation(fig, img, masks, flows[0], omni=omni)
+        fig.savefig(os.path.join(savedir,basename + '_cp_output' + suffix + '.png'), dpi=300)
         plt.close(fig)
 
-    if masks.ndim < 3 and outlines: 
-        mask_outlines = utils.outlines_list(masks)
-        outlines_to_text(base, mask_outlines)
-
+    # ImageJ txt outline files 
+    if masks.ndim < 3 and save_txt:
+        check_dir(txtdir)
+        outlines = utils.outlines_list(masks)
+        outlines_to_text(os.path.join(txtdir,basename), outlines)
+    
+    # RGB outline images
+    if masks.ndim < 3 and save_outlines: 
+        check_dir(outlinedir) 
+        outlines = utils.masks_to_outlines(masks)
+        outX, outY = np.nonzero(outlines)
+        img0 = transforms.normalize99(images,omni=omni)
+        if img0.shape[0] < 4:
+            img0 = np.transpose(img0, (1,2,0))
+        if img0.shape[-1] < 3 or img0.ndim < 3:
+            img0 = plot.image_to_rgb(img0, channels=channels)
+        else:
+            if img0.max()<=50.0:
+                img0 = np.uint8(np.clip(img0*255, 0, 1))
+        imgout= img0.copy()
+        imgout[outX, outY] = np.array([255,0,0]) #pure red 
+        imsave(os.path.join(outlinedir, basename + '_outlines' + suffix + '.png'),  imgout)
+    
+    # ncolor labels (ready for color map application)
+    if masks.ndim < 3 and save_ncolor:
+        check_dir(ncolordir)
+        #convert masks to minimal n-color reresentation 
+        imsave(os.path.join(ncolordir, basename + '_cp_ncolor_masks' + suffix + '.png'),
+               omnipose.utils.ncolorlabel(masks))
+    
+    # save RGB flow picture
+    if masks.ndim < 3 and save_flows:
+        check_dir(flowdir)
+        imsave(os.path.join(flowdir, basename + '_flows' + suffix + '.tif'), (flows[0]*(2**16 - 1)).astype(np.uint16))
+        #save full flow data
+        imsave(os.path.join(flowdir, basename + '_dP' + suffix + '.tif'), flows[1]) 
+    
 def save_server(parent=None, filename=None):
     """ Uploads a *_seg.npy file to the bucket.
     
@@ -394,4 +476,5 @@ def save_server(parent=None, filename=None):
             "File {} uploaded to {}.".format(
                 source_file_name, destination_blob_name
             )
-        )
\ No newline at end of file
+        )
+
