@@ -97,7 +97,7 @@ class Cellpose():
         if nuclear:
             self.diam_mean = 17. 
         elif bacterial:
-            self.diam_mean = 0.
+            #self.diam_mean = 0.
             net_avg = False # No bacterial or omni models have additional models
         
         if not net_avg:
@@ -376,7 +376,7 @@ class CellposeModel(UnetModel):
             if nuclear:
                 self.diam_mean = 17. 
             elif bacterial:
-                self.diam_mean = 0.
+                #self.diam_mean = 0.
                 net_avg = False #'bact' model also has no 1,2,3
 
             # set omni flag to true if the name contains it
@@ -393,6 +393,7 @@ class CellposeModel(UnetModel):
                 if params is not None:
                     residual_on, style_on, concatenation = params 
                 self.omni = 'omni' in os.path.splitext(Path(pretrained_model_string).name)[0]
+        
         # must have four classes for omnipose models
         # Note that omni can still be used independently for evaluation to 'mix and match'
         #would be better just to read from the model 
@@ -716,8 +717,7 @@ class CellposeModel(UnetModel):
                                                       use_gpu=self.gpu, device=self.device, nclasses=self.nclasses)
             else:
                 masks = np.zeros((nimg, shape[1], shape[2]), np.uint16)
-                p = np.zeros((2, nimg, shape[1], shape[2]) if not resample else dP.shape, np.uint16)
-#                 p = np.zeros(dP.shape, np.uint16)
+                p = np.zeros(dP.shape, np.uint16)
 
                 tr = [[]]*nimg # trace may not work correctly with multiple images currently, still need to test it 
                 resize = [shape[1], shape[2]] if not resample else None
